@@ -95,10 +95,14 @@ class Pipe( object ):
         """
         result = []
         self.pause_on_silence = pause_on_silence
-        for item in itertools.ifilter( bool, self ):
-            if item:
-                result.append( item )
-        return "".join( result )
+        try:
+            for item in itertools.ifilter( bool, self ):
+                if item:
+                    result.append( item )
+            return "".join( result )
+        except Exception, err:
+            err.output = result
+            raise
     def append( self, process ):
         """Add the given PipeComponent to this pipe (note: does not connect stdin/stdout)"""
         assert isinstance( process, PipeComponent ), process
@@ -410,7 +414,11 @@ class Process( PipeComponent ):
         if exitcode < 0:
             self.pipe.kill()
             exitcode = self.pipe.poll()
-        if exitcode not in self.good_exit:
+        if exitcode is None:
+            log.error(
+                "Process %s appears not to have exited properly", self 
+            )
+        elif exitcode not in self.good_exit:
             err = ProcessError( 
                 "Process %s returned error code %s"%( self.command, exitcode, ),
             )
@@ -567,7 +575,11 @@ def close( fh ):
     if hasattr( fh, 'close'):
         errcode = fh.close()
     else:
-        errcode = os.close( fileno(fh) )
+        fn = fileno( fh )
+        if fn and isinstance( fn, (int,long)):
+            errcode = os.close( fileno(fh) )
+        else:
+            errcode = 0
     if errcode:
         raise RuntimeError( "Child process returned error code", errcode )
 def by_line( iterable ):
